/* -*-	Mode:C++ -*- */

/*
 * blob.hh
 * Copyright (C) 1999 by John Heidemann
 * $Id: blob.hh,v 1.10 1999/05/21 04:08:35 johnh Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */


#ifndef lavaps_blob_h
#define lavaps_blob_h

class process_view;

class blob {
private:
	blob(const blob& tb);
	blob& operator=(const blob& tb);

protected:
	process_view *pv_;

	double H_, S_, B_;
	int size_;

	int x_step_;
	int x_;
	int *y_lows_, *y_highs_;
	int num_;
	int alloced_num_;

	int sign_;
	int reached_limit_;

	void resize_int_array(int **iap, int on, int nn);
	void adjust_num(int new_num);
	inline bool valid_num(int n) { return n >= 0 && n < num_; }

	const int DELTA_LIMIT = 5;
	enum blob_op { GROW_OP, MOVE_OP };

	blob *superior();

	void partial_op(blob_op op, int delta);
	virtual void op(blob_op op, int delta);

	void shift_to_right();
	void shift_to_left();
	void widen_on_right();
	void widen_on_left();
	bool narrow_on_right();
	bool narrow_on_left();

	int advance_rank(blob_op op, int num, int delta);
	int find_rank_midpoint(int num);
	int ranks_overlap(int a, int b);

	void reverse_direction();
public:
	blob(process_view *pv);
	virtual ~blob();

	const int X_MAX = 100, Y_MAX = 200, INITIAL_SLOP = 10;

	static blob *create_real_blob(process_view *pv); // instantiate a back-end blob

	void grow(int delta) { op(GROW_OP, delta); };
	void move(int delta) { op(MOVE_OP, delta); };
	void print();

	virtual void set_hsb(double h, double s, double b);
	virtual void set_size(int size);
	virtual void set_position(int x, int y);

	void info_to_buf(char *buf, int len);
};

#endif /* lavaps_blob_h */

