/* -*-	Mode:C++ -*- */

/*
 * process_view.h
 * Copyright (C) 1999 by John Heidemann
 * $Id: process_view.hh,v 1.16 1999/05/11 03:38:56 johnh Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */


#ifndef lavaps_process_view_h
#define lavaps_process_view_h

#include <unistd.h>  // pid_t

#include "blob.hh"
#include "change_tracking.hh"

class process_model;

/*
 * process_view: a process-level representation of 
 * how we'll render the process.
 * This is independent of how we'll actually render it
 * (i.e., Tcl, GNOME, etc.).
 */
class process_view {
private:
	process_view(const process_view& tb);
	process_view& operator=(const process_view& tb);

protected:
	process_model *pm_;

	blob *b_;

	// static stuff:
	double H_, S_, B_;
	// int start_x_, start_y_;

	change_tracking<int> size_;
	change_tracking<int> time_;
	change_tracking<int> age_;

	int scale_size(int size);
	int scale_time(int time);

	void reset_color();
	void init_position();
public:
	process_view(process_model *pm);
	~process_view();
	void update();
	void info_to_buf(char *buf, int buflen);

	blob *get_blob() { return b_; }
	process_view *superior();
};


#endif /* lavaps_process_view_h */

